@@ -11,3 +11,7 @@
 #undef HAVE_XAUTH
 
 #undef socklen_t
+
+#undef HAVE_X_WINDOWS
+
+#undef _GNU_SOURCE
