@@ -48,27 +48,18 @@ Boston, MA 02111-1307, USA.
  * ../etc/gnuserv.README relative to the directory containing this file)
  */
 
+#ifdef  CYGWIN
+extern void cygwin_conv_to_posix_path(const char *path, char *posix_path);
+#endif
+
 #include "gnuserv.h"
 
 char gnuserv_version[] = "gnuclient version " GNUSERV_VERSION;
 
 #include "getopt.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <sys/types.h>
 #include <sysfile.h>
 
-#ifdef HAVE_STRING_H
-#include <string.h>
-#endif /* HAVE_STRING_H */
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif /* HAVE_UNISTD_H */
-
-#include <signal.h>
-
 #if !defined(SYSV_IPC) && !defined(UNIX_DOMAIN_SOCKETS) && \
     !defined(INTERNET_DOMAIN_SOCKETS)
 int
@@ -82,7 +73,7 @@ main (int argc, char *argv[])
 } /* main */
 #else /* SYSV_IPC || UNIX_DOMAIN_SOCKETS || INTERNET_DOMAIN_SOCKETS */
 
-static char cwd[MAXPATHLEN+2];	/* current working directory when calculated */
+static char cwd[PATH_MAX+2];	/* current working directory when calculated */
 static char *cp = NULL;		/* ptr into valid bit of cwd above */
 
 static pid_t emacs_pid;			/* Process id for emacs process */
@@ -107,7 +98,7 @@ tell_emacs_to_resume (int sig)
   signal(SIGCONT, tell_emacs_to_resume);
 #endif
 
-  connect_type = make_connection (NULL, (u_short) 0, &s);
+  connect_type = make_connection (NULL, 0, &s);
 
   sprintf(buffer,"(gnuserv-eval '(resume-pid-console %d))", (int)getpid());
   send_string(s, buffer);
@@ -160,11 +151,11 @@ get_current_working_directory (void)
 {
   if (cp == NULL)
     {				/* haven't calculated it yet */
-#ifdef BSD
+#ifdef HAVE_GETCWD
+      if (getcwd (cwd,PATH_MAX) == NULL)
+#else
       if (getwd (cwd) == 0)
-#else /* !BSD */
-      if (getcwd (cwd,MAXPATHLEN) == NULL)
-#endif /* !BSD */
+#endif /* HAVE_GETCWD */
 	{
 	  perror (progname);
 	  fprintf (stderr, "%s: unable to get current working directory\n",
@@ -193,35 +184,27 @@ filename_expand (char *fullpath, char *filename)
   /* fullpath - returned full pathname */
   /* filename - filename to expand */
 {
-  int len;
+#ifdef  CYGWIN
+  char cygwinFilename[PATH_MAX+1];
+  extern void cygwin_conv_to_posix_path(const char *, char *);
+#endif
 
+  int len;
   fullpath[0] = '\0';
 
+#ifdef  CYGWIN
+  /*
+    If we're in cygwin, just convert it and let the unix stuff handle it.
+  */
+  cygwin_conv_to_posix_path(filename, cygwinFilename);
+  filename = cygwinFilename;
+#endif
+
   if (filename[0] && filename[0] == '/')
      {
        /* Absolute (unix-style) pathname.  Do nothing */
        strcat (fullpath, filename);
      }
-#ifdef  CYGWIN
-  else if (filename[0] && filename[0] == '\\' &&
-           filename[1] && filename[1] == '\\')
-    {
-      /* This path includes the server name (something like
-         "\\server\path"), so we assume it's absolute.  Do nothing to
-         it. */
-      strcat (fullpath, filename);
-    }
-  else if (filename[0] &&
-           filename[1] && filename[1] == ':' &&
-           filename[2] && filename[2] == '\\')
-    {
-      /* Absolute pathname with drive letter.  Convert "<drive>:"
-         to "//<drive>/". */
-      strcat (fullpath, "//");
-      strncat (fullpath, filename, 1);
-      strcat (fullpath, &filename[2]);
-    }
-#endif
   else
     {
       /* Assume relative Unix style path.  Get the current directory
@@ -317,8 +300,8 @@ int
 main (int argc, char *argv[])
 {
   int starting_line = 1;	/* line to start editing at */
-  char command[MAXPATHLEN+50];	/* emacs command buffer */
-  char fullpath[MAXPATHLEN+1];	/* full pathname to file */
+  char command[PATH_MAX+50];	/* emacs command buffer */
+  char fullpath[PATH_MAX+1];	/* full pathname to file */
   char *eval_form = NULL;	/* form to evaluate with `-eval' */
   char *eval_function = NULL;	/* function to evaluate with `-f' */
   char *load_library = NULL;	/* library to load */
@@ -338,10 +321,10 @@ main (int argc, char *argv[])
   char *hostarg = NULL;		/* remote hostname */
   char *remotearg;
   char thishost[HOSTNAMSZ];	/* this hostname */
-  char remotepath[MAXPATHLEN+1]; /* remote pathname */
+  char remotepath[PATH_MAX+1]; /* remote pathname */
   int rflg = 0;			/* pathname given on cmdline */
   char *portarg;
-  u_short port = 0;		/* port to server */
+  unsigned short port = 0;	/* port to server */
 #endif /* INTERNET_DOMAIN_SOCKETS */
   char *tty = NULL;
   char buffer[GSERV_BUFSZ + 1];	/* buffer to read pid */
@@ -358,12 +341,19 @@ main (int argc, char *argv[])
   else
     progname = argv[0];
 
+#ifdef WIN32_NATIVE
+  tmpdir = getenv ("TEMP");
+  if (!tmpdir)
+    tmpdir = getenv ("TMP");
+  if (!tmpdir)
+    tmpdir = "c:\\";
+#else
 #ifdef USE_TMPDIR
   tmpdir = getenv ("TMPDIR");
 #endif
   if (!tmpdir)
     tmpdir = "/tmp";
-
+#endif /* WIN32_NATIVE */
   display = getenv ("DISPLAY");
   if (display)
     display = my_strdup (display);
@@ -476,7 +466,7 @@ main (int argc, char *argv[])
 	       progname);
       exit (1);
     }
-#ifdef INTERNET_DOMAIN_SOCKETS
+#if defined(INTERNET_DOMAIN_SOCKETS)
   if (suppress_windows_system && hostarg)
     {
       fprintf (stderr, "%s: Remote editing is available only on X\n",
@@ -484,14 +474,13 @@ main (int argc, char *argv[])
       exit (1);
     }
 #endif
-
   *result = '\0';
   if (eval_function || eval_form || load_library)
     {
 #if defined(INTERNET_DOMAIN_SOCKETS)
       connect_type = make_connection (hostarg, port, &s);
 #else
-      connect_type = make_connection (NULL, (u_short) 0, &s);
+      connect_type = make_connection (NULL, 0, &s);
 #endif
       sprintf (command, "(gnuserv-eval%s '(progn ", quick ? "-quickly" : "");
       send_string (s, command);
@@ -529,7 +518,7 @@ main (int argc, char *argv[])
 #if defined(INTERNET_DOMAIN_SOCKETS)
       connect_type = make_connection (hostarg, port, &s);
 #else
-      connect_type = make_connection (NULL, (u_short) 0, &s);
+      connect_type = make_connection (NULL, 0, &s);
 #endif
       sprintf (command, "(gnuserv-eval%s '(progn ", quick ? "-quickly" : "");
       send_string (s, command);
@@ -563,7 +552,7 @@ main (int argc, char *argv[])
 #if defined(INTERNET_DOMAIN_SOCKETS)
 	  connect_type = make_connection (hostarg, port, &s);
 #else
-	  connect_type = make_connection (NULL, (u_short) 0, &s);
+	  connect_type = make_connection (NULL, 0, &s);
 #endif
 	  send_string (s, "(gnuserv-eval '(emacs-pid))");
 	  send_string (s, EOT_STR);
@@ -579,7 +568,7 @@ main (int argc, char *argv[])
 #ifdef SYSV_IPC
 	  if (connect_type == (int) CONN_IPC)
 	    disconnect_from_ipc_server (s, msgp, FALSE);
-#endif /* SYSV_IPC */
+#endif /* !SYSV_IPC */
 
 	  emacs_pid = (pid_t)atol(buffer);
 	  initialize_signals();
@@ -588,7 +577,7 @@ main (int argc, char *argv[])
 #if defined(INTERNET_DOMAIN_SOCKETS)
       connect_type = make_connection (hostarg, port, &s);
 #else
-      connect_type = make_connection (NULL, (u_short) 0, &s);
+      connect_type = make_connection (NULL, 0, &s);
 #endif
 
 #ifdef INTERNET_DOMAIN_SOCKETS
@@ -644,9 +633,17 @@ main (int argc, char *argv[])
 	}
       else /* !suppress_windows_system */
 	{
-	  if (display)
+	  if (0)
+	    ;
+#ifdef HAVE_X_WINDOWS
+	  else if (display)
 	    sprintf (command, "(gnuserv-edit-files '(x %s) '(",
 		     clean_string (display));
+#endif
+#ifdef HAVE_GTK
+	  else if (display)
+	    strcpy (command, "(gnuserv-edit-files '(gtk nil) '(");
+#endif
 #ifdef HAVE_MS_WINDOWS
 	  else
 	    sprintf (command, "(gnuserv-edit-files '(mswindows nil) '(");
